@@ -3,7 +3,6 @@
 # Email:  ksible *at* outlook *dot* com
 
 # pylint: disable = attribute-defined-outside-init, protected-access, exec-used
-from token import ISNONTERMINAL
 from sympy import Function, Derivative, Symbol, Integer, Rational, Float, Pow, Add
 from sympy import sin, cos, tan, sinh, cosh, tanh, asin, acos, atan, asinh, acosh, atanh
 from sympy import pi, exp, log, sqrt, expand, diff, srepr
@@ -14,7 +13,7 @@ from collections import OrderedDict
 from inspect import currentframe
 import re, sys, math, warnings
 
-class Lexer:
+class Lexer(object):
     """ LaTeX Lexer
 
         The following class will tokenize a LaTeX sentence for parsing.
@@ -31,11 +30,11 @@ class Lexer:
             r'\\[uU]psilon', r'\\[pP]hi', r'\\[cC]hi', r'\\[pP]si', r'\\[oO]mega', r'\\varepsilon', r'\\varkappa',
             r'\\varphi', r'\\varpi', r'\\varrho', r'\\varsigma', r'\\vartheta', r'[a-zA-Z]'))
         self.token_dict = [
+            ('EOL',             r'\r?\n'),
             ('WHITESPACE',      r'\s+'),
             ('STRING',          r'\"[^\"]*\"'),
             ('GROUP',           r'\<[0-9]+(\.{2})?\>'),
             ('DIMENSION',       r'[2-9][0-9]*D'),
-            ('VARIABLE',        r'\'[a-zA-Z][_a-zA-Z]*\''),
             ('RATIONAL',        r'\-?[0-9]+\/\-?[1-9][0-9]*'),
             ('DECIMAL',         r'\-?[0-9]+\.[0-9]+'),
             ('INTEGER',         r'\-?[0-9]+'),
@@ -69,13 +68,13 @@ class Lexer:
             ('NLOG_CMD',        r'\\ln|\\log'),
             ('TRIG_CMD',        r'\\sinh|\\cosh|\\tanh|\\sin|\\cos|\\tan'),
             ('VARDEF_MACRO',    r'vardef'),
-            ('KEYDEF_MACRO',    r'keydef'),
+            ('ATTRIB_MACRO',    r'attrib'),
             ('ASSIGN_MACRO',    r'assign'),
             ('IGNORE_MACRO',    r'ignore'),
             ('PARSE_MACRO',     r'parse'),
             ('SREPL_MACRO',     r'srepl'),
             ('INDEX_KWRD',      r'index'),
-            ('BASIS_KWRD',      r'basis'),
+            ('COORD_KWRD',      r'coord'),
             ('DIFF_TYPE',       r'diff_type'),
             ('CONSTANT',        r'const'),
             ('KRONECKER',       r'kron'),
@@ -97,7 +96,7 @@ class Lexer:
         self.regex = re.compile('|'.join(['(?P<%s>%s)' % pattern for pattern in self.token_dict]))
         self.token_dict = dict(self.token_dict)
 
-    def initialize(self, sentence, position=0):
+    def initialize(self, sentence, position=0, whitespace=False):
         """ Initialize Lexer
 
             :arg: sentence (raw string)
@@ -108,6 +107,7 @@ class Lexer:
         self.lexeme   = None
         self.marker   = None
         self.index    = position
+        self._whitespace = whitespace
 
     def tokenize(self):
         """ Tokenize Sentence
@@ -120,7 +120,7 @@ class Lexer:
                 raise ParseError('unexpected \'%s\' at position %d' %
                     (self.sentence[self.index], self.index), self.sentence, self.index)
             self.index = token.end()
-            if token.lastgroup != 'WHITESPACE':
+            if self.whitespace or token.lastgroup not in ('WHITESPACE', 'EOL'):
                 self.lexeme = token.group()
                 yield token.lastgroup
 
@@ -148,9 +148,35 @@ class Lexer:
         """ Reset Token Iterator """
         if not self.sentence:
             raise RuntimeError('cannot reset uninitialized lexer')
-        self.initialize(self.sentence, self.marker if index is None else index)
+        self.initialize(self.sentence, self.marker if index is None else index, self.whitespace)
         self.lex()
 
+    @property
+    def whitespace(self):
+        return self._whitespace
+
+    @whitespace.setter
+    def whitespace(self, flag):
+        if not flag:
+            while self.token in ('WHITESPACE', 'EOL'):
+                self.lex()
+        self._whitespace = flag
+
+    def new_context(self):
+        return self.LexerContext(self)
+
+    class LexerContext():
+
+        def __init__(self, lexer):
+            self.lexer = lexer
+            self.state = (lexer.sentence, lexer.mark(), lexer.whitespace)
+
+        def __enter__(self): return
+
+        def __exit__(self, exc_type, exc_value, exc_tb):
+            self.lexer.initialize(*self.state)
+            self.lexer.lex()
+
 class Parser:
     """ LaTeX Parser
 
@@ -159,17 +185,17 @@ class Parser:
         # LaTeX Extended BNF Grammar:
         # <LATEX>         -> ( <ALIGN> | '%' <MACRO> | <ASSIGNMENT> ) { [ <RETURN> ] ( <ALIGN> | '%' <MACRO> | <ASSIGNMENT> ) }*
         # <ALIGN>         -> <OPENING> ( '%' <MACRO> | <ASSIGNMENT> ) { [ <RETURN> ] ( '%' <MACRO> | <ASSIGNMENT> ) }* <CLOSING>
-        #     <MACRO>     -> <PARSE> | <SREPL> | <VARDEF> | <KEYDEF> | <ASSIGN> | <IGNORE>
-        #     <PARSE>     -> <PARSE_MACRO> <ASSIGNMENT> { ',' <ASSIGNMENT> }*
+        #     <MACRO>     -> <PARSE> | <SREPL> | <VARDEF> | <ATTRIB> | <ASSIGN> | <IGNORE>
+        #     <PARSE>     -> <PARSE_MACRO> <ASSIGNMENT> { ',' <ASSIGNMENT> }* '\\'
         #     <SREPL>     -> <SREPL_MACRO> [ '-' <PERSIST> ] <STRING> <ARROW> <STRING> { ',' <STRING> <ARROW> <STRING> }*
-        #     <VARDEF>    -> <VARDEF_MACRO> { '-' ( <OPTION> | <ZERO> ) }* <VARIABLE> { ',' <VARIABLE> }* [ '(' <DIMENSION> ')' ]
-        #     <KEYDEF>    -> <KEYDEF_MACRO> ( <BASIS_KWRD> ( <BASIS> | <DEFAULT> ) | <INDEX_KWRD> ( <INDEX> | <DEFAULT> ) )
+        #     <VARDEF>    -> <VARDEF_MACRO> { '-' ( <OPTION> | <ZERO> ) }* <VARIABLE> [ '::' <DIMENSION> ] { ',' <VARIABLE> [ '::' <DIMENSION> ] }*
+        #     <ATTRIB>    -> <ATTRIB_MACRO> ( <COORD_KWRD> ( <COORD> | <DEFAULT> ) | <INDEX_KWRD> ( <INDEX> | <DEFAULT> ) )
         #     <ASSIGN>    -> <ASSIGN_MACRO> { '-' <OPTION> }* <VARIABLE> { ',' <VARIABLE> }*
         #     <IGNORE>    -> <IGNORE_MACRO> <STRING> { ',' <STRING> }*
         #     <OPTION>    -> <CONSTANT> | <KRONECKER> | <METRIC> [ '=' <VARIABLE> ] | <WEIGHT> '=' <NUMBER>
         #                     | <DIFF_TYPE> '=' <DIFF_OPT> | <SYMMETRY> '=' <SYM_OPT>
-        #     <BASIS>     -> <BASIS_KWRD> <LBRACK> <SYMBOL> [ ',' <SYMBOL> ]* <RBRACK>
-        #     <INDEX>     -> ( <LETTER> | '[' <LETTER> '-' <LETTER> ']' ) '(' <DIMENSION> ')'
+        #     <COORD>     -> <COORD_KWRD> <LBRACK> <SYMBOL> [ ',' <SYMBOL> ]* <RBRACK>
+        #     <INDEX>     -> ( <LETTER> | '[' <LETTER> '-' <LETTER> ']' ) '::' <DIMENSION>
         # <ASSIGNMENT>    -> <OPERATOR> = <EXPRESSION>
         # <EXPRESSION>    -> <TERM> { ( '+' | '-' ) <TERM> }*
         # <TERM>          -> <FACTOR> { [ '/' ] <FACTOR> }*
@@ -193,6 +219,7 @@ class Parser:
         # <INDEXING_2>    -> <LETTER> | <INTEGER> | '{' <INDEXING_1> '}'
         # <INDEXING_3>    -> <INDEXING_2> | '{' { <INDEXING_1> }+ '}'
         # <INDEXING_4>    -> <INDEXING_2> | '{' ( ',' | ';' ) { <INDEXING_1> }+ | { <INDEXING_1> }+ [ ( ',' | ';' ) { <INDEXING_1> }+ ] '}'
+        # <VARIABLE>      -> <LETTER> { <LETTER> | <UNDERSCORE> }*
         # <NUMBER>        -> <RATIONAL> | <DECIMAL> | <INTEGER> | <PI>
 
     _namespace, _property = OrderedDict(), {}
@@ -216,7 +243,7 @@ class Parser:
         if reset: Parser._namespace.clear()
         Parser._property['dimension'] = 3
         Parser._property['srepl'] = []
-        Parser._property['basis'] = CoordinateSystem('x')
+        Parser._property['coord'] = CoordinateSystem('x')
         Parser._property['index'] = {i: Parser._property['dimension']
             for i in (chr(i) for i in range(105, 123))} # 105 -> 97
         Parser._property['ignore'] = ['\\left', '\\right', '{}', '&']
@@ -321,7 +348,7 @@ class Parser:
             else: self._assignment()
             if self.accept('RETURN'): pass
 
-    # <MACRO> -> <PARSE> | <SREPL> | <VARDEF> | <KEYDEF> | <ASSIGN> | <IGNORE>
+    # <MACRO> -> <PARSE> | <SREPL> | <VARDEF> | <ATTRIB> | <ASSIGN> | <IGNORE>
     def _macro(self):
         macro = self.lexer.lexeme
         if self.peek('PARSE_MACRO'):
@@ -330,8 +357,8 @@ class Parser:
             self._srepl()
         elif self.peek('VARDEF_MACRO'):
             self._vardef()
-        elif self.peek('KEYDEF_MACRO'):
-            self._keydef()
+        elif self.peek('ATTRIB_MACRO'):
+            self._attrib()
         elif self.peek('ASSIGN_MACRO'):
             self._assign()
         elif self.peek('IGNORE_MACRO'):
@@ -341,12 +368,13 @@ class Parser:
             raise ParseError('unsupported macro \'%s\' at position %d' %
                 (macro, position), sentence, position)
 
-    # <PARSE> -> <PARSE_MACRO> <ASSIGNMENT> { ',' <ASSIGNMENT> }*
+    # <PARSE> -> <PARSE_MACRO> <ASSIGNMENT> { ',' <ASSIGNMENT> }* '\\'
     def _parse(self):
         self.expect('PARSE_MACRO')
         self._assignment()
         while self.accept('COMMA'):
             self._assignment()
+        self.expect('RETURN')
 
     # <SREPL> -> <SREPL_MACRO> [ '-' <PERSIST> ] <STRING> <ARROW> <STRING> { ',' <STRING> <ARROW> <STRING> }*
     def _srepl(self):
@@ -421,15 +449,19 @@ class Parser:
             self.lexer.reset(); self.lexer.lex()
             if not self.accept('COMMA'): break
 
-    # <VARDEF> -> <VARDEF_MACRO> { '-' ( <OPTION> | <ZERO> ) }* <VARIABLE> { ',' <VARIABLE> }* [ '(' <DIMENSION> ')' ]
+    # <VARDEF> -> <VARDEF_MACRO> { '-' ( <OPTION> | <ZERO> ) }* <VARIABLE> [ '::' <DIMENSION> ] { ',' <VARIABLE> [ '::' <DIMENSION> ] }*
     def _vardef(self):
+        self.lexer.whitespace = True
         self.expect('VARDEF_MACRO')
         diff_type, symmetry = None, None
         metric, weight = None, None
         zero = False
+        self.accept('WHITESPACE')
         while self.accept('MINUS'):
             zero = self.accept('ZERO')
-            if zero: continue
+            if zero:
+                self.accept('WHITESPACE')
+                continue
             option, value = self._option().split('<>')
             if option == 'diff_type':
                 diff_type = value
@@ -439,19 +471,16 @@ class Parser:
                 metric = value
             elif option == 'weight':
                 weight = value
+            self.accept('WHITESPACE')
         while True:
-            symbol = self.lexer.lexeme[1:-1]
-            if not symbol and self.peek('VARIABLE'):
-                sentence, position = self.lexer.sentence, self.lexer.mark()
-                raise ParseError('empty variable at position %d' %
-                    position, sentence, position)
-            self.expect('VARIABLE')
+            symbol = self._variable()
             dimension = self._property['dimension']
-            if self.accept('LPAREN'):
+            if self.accept('COLON'):
+                self.expect('COLON')
                 dimension = self.lexer.lexeme[:-1]
                 self.expect('DIMENSION')
+                self.accept('WHITESPACE')
                 dimension = int(dimension)
-                self.expect('RPAREN')
             if symmetry == 'const':
                 self._namespace[symbol] = Function('Constant')(Symbol(symbol, real=True))
                 self.state.add(symbol)
@@ -467,22 +496,20 @@ class Parser:
             if symmetry == 'metric':
                 diacritic = next(diacritic for diacritic in ('bar', 'hat', 'tilde', '') if diacritic in symbol)
                 self._property['metric'][diacritic] = re.split(diacritic if diacritic else r'[UD]', symbol)[0]
-                # connection = 'Gamma' + diacritic + 'UDD'
-                # if connection in self._namespace:
-                #     del self._namespace[connection]
-                sentence, position = self.lexer.sentence, self.lexer.mark()
-                self.parse_latex(self._generate_metric(symbol, dimension, diacritic, diff_type))
-                self.lexer.initialize(sentence, position)
-                self.lexer.lex()
+                with self.lexer.new_context():
+                    self.parse_latex(self._generate_metric(symbol, dimension, diacritic, diff_type))
             if not self.accept('COMMA'): break
+            self.accept('WHITESPACE')
+        self.accept('EOL')
+        self.lexer.whitespace = False
 
-    # <KEYDEF> -> <KEYDEF_MACRO> ( <BASIS_KWRD> ( <BASIS> | <DEFAULT> ) | <INDEX_KWRD> ( <INDEX> | <DEFAULT> ) )
-    def _keydef(self):
-        self.expect('KEYDEF_MACRO')
-        if self.accept('BASIS_KWRD'):
+    # <ATTRIB> -> <ATTRIB_MACRO> ( <COORD_KWRD> ( <COORD> | <DEFAULT> ) | <INDEX_KWRD> ( <INDEX> | <DEFAULT> ) )
+    def _attrib(self):
+        self.expect('ATTRIB_MACRO')
+        if self.accept('COORD_KWRD'):
             if self.accept('DEFAULT'):
-                self._property['basis'] = CoordinateSystem('x')
-            else: self._basis()
+                self._property['coord'] = CoordinateSystem('x')
+            else: self._coord()
         elif self.accept('INDEX_KWRD'):
             if self.accept('DEFAULT'):
                 self._property['index'] = {i: self._property['dimension']
@@ -495,9 +522,11 @@ class Parser:
 
     # <ASSIGN> -> <ASSIGN_MACRO> { '-' <OPTION> }* <VARIABLE> { ',' <VARIABLE> }*
     def _assign(self):
+        self.lexer.whitespace = True
         self.expect('ASSIGN_MACRO')
         diff_type, symmetry = None, None
         metric, weight = None, None
+        self.accept('WHITESPACE')
         while self.accept('MINUS'):
             option, value = self._option().split('<>')
             if option == 'diff_type':
@@ -508,13 +537,9 @@ class Parser:
                 metric = value
             elif option == 'weight':
                 weight = value
+            self.accept('WHITESPACE')
         while True:
-            symbol = self.lexer.lexeme[1:-1]
-            if not symbol and self.peek('VARIABLE'):
-                sentence, position = self.lexer.sentence, self.lexer.mark()
-                raise ParseError('empty variable at position %d' %
-                    position, sentence, position)
-            self.expect('VARIABLE')
+            symbol = self._variable()
             if symbol not in self._namespace:
                 rank = 0
                 for symbol in re.split(r'_d|_dup|_cd|_ld', symbol):
@@ -556,13 +581,8 @@ class Parser:
                 symmetry = tensor.symmetry = 'sym01'
                 diacritic = next(diacritic for diacritic in ('bar', 'hat', 'tilde', '') if diacritic in symbol)
                 self._property['metric'][diacritic] = re.split(diacritic if diacritic else r'[UD]', symbol)[0]
-                # connection = 'Gamma' + diacritic + 'UDD'
-                # if connection in self._namespace:
-                #     del self._namespace[connection]
-                sentence, position = self.lexer.sentence, self.lexer.mark()
-                self.parse_latex(self._generate_metric(symbol, dimension, diacritic, diff_type))
-                self.lexer.initialize(sentence, position)
-                self.lexer.lex()
+                with self.lexer.new_context():
+                    self.parse_latex(self._generate_metric(symbol, dimension, diacritic, diff_type))
             base_symbol = re.split(r'_d|_dup|_cd|_ld', symbol)[0]
             if base_symbol and diff_type:
                 rank = 0
@@ -577,6 +597,9 @@ class Parser:
                     function = Function('Tensor')(Symbol(base_symbol, real=True))
                     self._define_tensor(Tensor(function, diff_type=diff_type))
             if not self.accept('COMMA'): break
+            self.accept('WHITESPACE')
+        self.accept('EOL')
+        self.lexer.whitespace = False
 
     # <IGNORE> -> <IGNORE_MACRO> <STRING> { ',' <STRING> }*
     def _ignore(self):
@@ -592,6 +615,7 @@ class Parser:
                 self.lexer.sentence = sentence[:position] + sentence[position:].replace(string, '')
             if not self.accept('COMMA'): break
         self.lexer.reset(); self.lexer.lex()
+        self.accept('EOL')
 
     # <OPTION> -> <CONSTANT> | <KRONECKER> | <METRIC> [ '=' <VARIABLE> ] | <WEIGHT> '=' <NUMBER>
     #              | <DIFF_TYPE> '=' <DIFF_OPT> | <SYMMETRY> '=' <SYM_OPT>
@@ -602,8 +626,7 @@ class Parser:
             return 'symmetry<>kron'
         if self.accept('METRIC'):
             if self.accept('EQUAL'):
-                metric = self.lexer.lexeme[1:-1]
-                self.expect('VARIABLE')
+                metric = self._variable()
                 return 'metric<>' + metric
             return 'symmetry<>metric'
         if self.accept('WEIGHT'):
@@ -624,23 +647,23 @@ class Parser:
         raise ParseError('unexpected \'%s\' at position %d' %
             (sentence[position], position), sentence, position)
 
-    # <BASIS> -> <BASIS_KWRD> <LBRACK> <SYMBOL> [ ',' <SYMBOL> ]* <RBRACK>
-    def _basis(self):
+    # <COORD> -> <COORD_KWRD> <LBRACK> <SYMBOL> [ ',' <SYMBOL> ]* <RBRACK>
+    def _coord(self):
         self.expect('LBRACK')
-        del self._property['basis'][:]
+        del self._property['coord'][:]
         while True:
             symbol = self._strip(self._symbol())
-            if symbol in self._property['basis']:
+            if symbol in self._property['coord']:
                 sentence, position = self.lexer.sentence, self.lexer.mark()
-                raise ParseError('duplicate basis symbol \'%s\' at position %d' %
+                raise ParseError('duplicate coord symbol \'%s\' at position %d' %
                     (sentence[position], position), sentence, position)
-            self._property['basis'].append(Symbol(symbol, real=True))
+            self._property['coord'].append(Symbol(symbol, real=True))
             if not self.accept('COMMA'): break
         self.expect('RBRACK')
         if not self._property['dimension']:
-            self._property['dimension'] = len(self._property['basis'])
+            self._property['dimension'] = len(self._property['coord'])
 
-    # <INDEX> -> ( <LETTER> | '[' <LETTER> '-' <LETTER> ']' )  '(' <DIMENSION> ')'
+    # <INDEX> -> ( <LETTER> | '[' <LETTER> '-' <LETTER> ']' )  '::' <DIMENSION>
     def _index(self):
         if self.accept('LBRACK'):
             index_1 = self._strip(self.lexer.lexeme)
@@ -653,11 +676,11 @@ class Parser:
         else:
             index = [self._strip(self.lexer.lexeme)]
             self.expect('LETTER')
-        self.expect('LPAREN')
+        self.expect('COLON')
+        self.expect('COLON')
         dimension = self.lexer.lexeme
         self.expect('DIMENSION')
         dimension = int(dimension[:-1])
-        self.expect('RPAREN')
         self._property['index'].update({i: dimension for i in index})
 
     # <ASSIGNMENT> -> <OPERATOR> = <EXPRESSION>
@@ -682,16 +705,14 @@ class Parser:
                 global_env[key] = global_env[key].structure
             if isinstance(global_env[key], Function('Constant')):
                 global_env[key] = global_env[key].args[0]
-        global_env['basis'] = self._property['basis']
+        global_env['coord'] = self._property['coord']
         exec('from sympy import *', global_env)
         # evaluate every implied summation and update namespace
         exec(LHS_RHS, global_env)
-        override = True
         symbol, indices = str(function.args[0]), function.args[1:]
         if any(isinstance(index, Integer) for index in indices):
             tensor = self._namespace[symbol]
             tensor.structure = global_env[symbol]
-            override = False
         else:
             diff_type = self._namespace[symbol].diff_type if symbol in self._namespace else None
             tensor = Tensor(function, dimension, structure=global_env[symbol],
@@ -1083,14 +1104,12 @@ class Parser:
                 equation[3] += '_{' + covdrv_index + '} '
             equation[0], equation[3] = equation[0] + latex, equation[3] + latex
             if location == 'RHS' and (self._property['vphantom'] or symbol not in self._namespace):
-                sentence, position = self.lexer.sentence, self.lexer.mark()
-                if index[1] == 'U':
-                    config = ' % assign -diff_type=dD \'' + symbol + '\''
-                    self.parse_latex(''.join(equation) + config)
-                else:
-                    self.parse_latex(self._generate_covdrv(function, index[0], symbol, diacritic))
-                self.lexer.initialize(sentence, position)
-                self.lexer.lex()
+                with self.lexer.new_context():
+                    if index[1] == 'U':
+                        config = ' % assign -diff_type=dD ' + symbol + '\n'
+                        self.parse_latex(''.join(equation) + config)
+                    else:
+                        self.parse_latex(self._generate_covdrv(function, index[0], symbol, diacritic))
         return expression
 
     # <LIEDRV> -> <LIE_SYM> '_' <SYMBOL> ( <OPERATOR> | <SUBEXPR> )
@@ -1207,42 +1226,40 @@ class Parser:
                     for suffix in product(*('UD' if i == 'U' else 'DU' for _, i in Tensor.indexing(function))):
                         symbol_RHS = base_symbol + ''.join(suffix)
                         if symbol_RHS in self._namespace:
-                            sentence, position = self.lexer.sentence, self.lexer.mark()
-                            diacritic = 'bar'   if 'bar'   in symbol \
-                                   else 'hat'   if 'hat'   in symbol \
-                                   else 'tilde' if 'tilde' in symbol \
-                                   else ''
-                            metric = self._namespace[symbol_RHS].metric if self._namespace[symbol_RHS].metric else \
-                                self._property['metric'][diacritic] + diacritic
-                            indexing_LHS = indexing_RHS = [str(index) for index in indexing]
-                            alphabet = (chr(97 + n) for n in range(26))
-                            for i, index in enumerate(indexing_LHS):
-                                if index in indexing_LHS[:i]:
-                                    indexing_LHS[i] = next(x for x in alphabet if x not in indexing_LHS)
-                            function_LHS = Function('Tensor')(function.args[0],
-                                *(Symbol(i) for i in indexing_LHS))
-                            latex = Tensor.latex_format(function_LHS) + ' = '
-                            for i, (idx, pos) in enumerate(Tensor.indexing(function_LHS)):
-                                if pos != suffix[i]:
-                                    indexing_RHS[i] = next(x for x in alphabet if x not in indexing_LHS)
-                                    if len(str(idx)) > 1:
-                                        idx = '_'.join('\\' + i if len(i) > 1 else i for i in str(idx).split('_'))
-                                    if pos == 'U':
-                                        latex += '\\text{%s}^{%s %s} ' % (metric, idx, indexing_RHS[i])
-                                    else:
-                                        latex += '\\text{%s}_{%s %s} ' % (metric, idx, indexing_RHS[i])
-                            latex += Tensor.latex_format(Function('Tensor')(Symbol(symbol_RHS, real=True), *indexing_RHS))
-                            diff_type = self._namespace[symbol_RHS].diff_type
-                            if diff_type or self._namespace[symbol_RHS].metric:
-                                latex += ' % assign '
-                                if diff_type:
-                                    latex += '-diff_type=' + diff_type + ' '
-                                if self._namespace[symbol_RHS].metric:
-                                    latex += '-metric=\'' + metric + '\' '
-                                latex += '\'' + symbol + '\''
-                            self.parse_latex(latex)
-                            self.lexer.initialize(sentence, position)
-                            self.lexer.lex()
+                            with self.lexer.new_context():
+                                diacritic = 'bar'   if 'bar'   in symbol \
+                                    else 'hat'   if 'hat'   in symbol \
+                                    else 'tilde' if 'tilde' in symbol \
+                                    else ''
+                                metric = self._namespace[symbol_RHS].metric if self._namespace[symbol_RHS].metric else \
+                                    self._property['metric'][diacritic] + diacritic
+                                indexing_LHS = indexing_RHS = [str(index) for index in indexing]
+                                alphabet = (chr(97 + n) for n in range(26))
+                                for i, index in enumerate(indexing_LHS):
+                                    if index in indexing_LHS[:i]:
+                                        indexing_LHS[i] = next(x for x in alphabet if x not in indexing_LHS)
+                                function_LHS = Function('Tensor')(function.args[0],
+                                    *(Symbol(i) for i in indexing_LHS))
+                                latex = Tensor.latex_format(function_LHS) + ' = '
+                                for i, (idx, pos) in enumerate(Tensor.indexing(function_LHS)):
+                                    if pos != suffix[i]:
+                                        indexing_RHS[i] = next(x for x in alphabet if x not in indexing_LHS)
+                                        if len(str(idx)) > 1:
+                                            idx = '_'.join('\\' + i if len(i) > 1 else i for i in str(idx).split('_'))
+                                        if pos == 'U':
+                                            latex += '\\text{%s}^{%s %s} ' % (metric, idx, indexing_RHS[i])
+                                        else:
+                                            latex += '\\text{%s}_{%s %s} ' % (metric, idx, indexing_RHS[i])
+                                latex += Tensor.latex_format(Function('Tensor')(Symbol(symbol_RHS, real=True), *indexing_RHS))
+                                diff_type = self._namespace[symbol_RHS].diff_type
+                                if diff_type or self._namespace[symbol_RHS].metric:
+                                    latex += ' % assign '
+                                    if diff_type:
+                                        latex += '-diff_type=' + diff_type + ' '
+                                    if self._namespace[symbol_RHS].metric:
+                                        latex += '-metric=' + metric + ' '
+                                    latex += symbol
+                                self.parse_latex(latex)
                             return function
                     raise ParseError('cannot index undefined tensor \'%s\' at position %d' %
                         (symbol, position), sentence, position)
@@ -1279,8 +1296,8 @@ class Parser:
         if self.accept('LETTER'):
             index = Symbol(lexeme, real=True) if not self.accept('UNDERSCORE') \
                 else Symbol('%s_%s' % (lexeme, self._indexing_2()), real=True)
-            return index if index not in self._property['basis'] \
-                else Integer(self._property['basis'].index(index))
+            return index if index not in self._property['coord'] \
+                else Integer(self._property['coord'].index(index))
         elif self.accept('INTEGER'):
             return Integer(lexeme)
         sentence, position = self.lexer.sentence, self.lexer.mark()
@@ -1292,8 +1309,8 @@ class Parser:
         lexeme = self._strip(self.lexer.lexeme)
         if self.accept('LETTER'):
             index = Symbol(lexeme, real=True)
-            return index if index not in self._property['basis'] \
-                else Integer(self._property['basis'].index(index))
+            return index if index not in self._property['coord'] \
+                else Integer(self._property['coord'].index(index))
         elif self.accept('INTEGER'):
             return Integer(lexeme)
         elif self.accept('LBRACE'):
@@ -1336,6 +1353,15 @@ class Parser:
             return indexing, order
         return [self._indexing_2()], order
 
+    # <VARIABLE> -> <LETTER> { <LETTER> | <UNDERSCORE> }*
+    def _variable(self):
+        variable = self.lexer.lexeme
+        self.expect('LETTER')
+        while self.peek('LETTER') or self.peek('UNDERSCORE') or self.peek('DIFF_OPT'):
+            variable += self.lexer.lexeme
+            self.lexer.lex()
+        return variable
+
     # <NUMBER> -> <RATIONAL> | <DECIMAL> | <INTEGER> | <PI>
     def _number(self):
         number = self.lexer.lexeme
@@ -1369,24 +1395,22 @@ class Parser:
             else ''
         tensor, index = self._namespace[symbol], str(index)
         symbol = symbol + ('' if suffix in symbol else suffix) + 'D'
-        sentence, position = self.lexer.sentence, self.lexer.mark()
-        if symbol not in self._namespace:
-            if location == 'RHS' and tensor.equation:
-                LHS, RHS = tensor.equation[0]
-                tree, idx_set = ExprTree(tensor.equation[1]), set()
-                for subtree in tree.preorder():
-                    subexpr = subtree.expr
-                    if subexpr.func == Function('Tensor'):
-                        idx_set.update(subexpr.args[1:])
-                idx_set = {str(i) for i in idx_set}
-                if index in idx_set:
-                    alphabet = (chr(97 + n) for n in range(26))
-                    index = next(x for x in alphabet if x not in idx_set)
-                if len(index) > 1:
-                    index = '_'.join('\\' + i if len(i) > 1 else i for i in index.split('_'))
-                self.parse_latex('\\partial_{%s} %s = \\partial_{%s} (%s)' % (index, LHS.strip(), index, RHS.strip()))
-                self.lexer.initialize(sentence, position)
-                self.lexer.lex()
+        with self.lexer.new_context():
+            if symbol not in self._namespace:
+                if location == 'RHS' and tensor.equation:
+                    LHS, RHS = tensor.equation[0]
+                    tree, idx_set = ExprTree(tensor.equation[1]), set()
+                    for subtree in tree.preorder():
+                        subexpr = subtree.expr
+                        if subexpr.func == Function('Tensor'):
+                            idx_set.update(subexpr.args[1:])
+                    idx_set = {str(i) for i in idx_set}
+                    if index in idx_set:
+                        alphabet = (chr(97 + n) for n in range(26))
+                        index = next(x for x in alphabet if x not in idx_set)
+                    if len(index) > 1:
+                        index = '_'.join('\\' + i if len(i) > 1 else i for i in index.split('_'))
+                    self.parse_latex('\\partial_{%s} %s = \\partial_{%s} (%s)' % (index, LHS.strip(), index, RHS.strip()))
         function = Function('Tensor')(Symbol(symbol, real=True), *indices)
         if symbol not in self._namespace:
             symmetry = 'nosym'
@@ -1501,7 +1525,7 @@ class Parser:
                         indexing.append((index, position))
             elif subexpr.func == Derivative:
                 for index, _ in subexpr.args[1:]:
-                    if index not in self._property['basis']:
+                    if index not in self._property['coord']:
                         if re.match(r'[a-zA-Z]+(?:_[0-9]+)?', str(index)):
                             indexing.append((index, 'D'))
         symbol_LHS = Tensor(LHS).symbol
@@ -1552,8 +1576,8 @@ class Parser:
                             raise ParseError('inconsistent dimension for index \'%s\'' %
                                 index, self.lexer.sentence)
                         idx_map[str(index)] = upper_bound
-                        if index not in self._property['basis']:
-                            derivative += ', (basis[%s], %s)' % (index, order)
+                        if index not in self._property['coord']:
+                            derivative += ', (coord[%s], %s)' % (index, order)
                             if re.match(r'[a-zA-Z]+(?:_[0-9]+)?', str(index)):
                                 indexing.append((index, 'D'))
                         else: derivative += ', (%s, %s)' % (index, order)
@@ -1625,7 +1649,7 @@ class Parser:
                 .format(symbol=symbol[:-2], inv_symbol=symbol.replace('U', 'D'), dimension=dimension,
                     factorial=math.factorial(dimension - 1), det_latex=det_latex, inv_latex=inv_latex)
             if diff_type:
-                latex_config += '\n' + r"% assign -diff_type={diff_type} '{symbol}det', '{inv_symbol}'" \
+                latex_config += '\n' + r"% assign -diff_type={diff_type} {symbol}det, {inv_symbol}" \
                     .format(diff_type=diff_type, symbol=symbol[:-2], inv_symbol=symbol.replace('U', 'D'))
             else: latex_config += r' \\'
         else:
@@ -1639,7 +1663,7 @@ class Parser:
                 .format(symbol=symbol[:-2], inv_symbol=symbol.replace('D', 'U'), dimension=dimension,
                     factorial=math.factorial(dimension - 1), det_latex=det_latex, inv_latex=inv_latex)
             if diff_type:
-                latex_config += '\n' + r"% assign -diff_type={diff_type} '{symbol}det', '{inv_symbol}'" \
+                latex_config += '\n' + r"% assign -diff_type={diff_type} {symbol}det, {inv_symbol}" \
                     .format(diff_type=diff_type, symbol=symbol[:-2], inv_symbol=symbol.replace('D', 'U'))
             else: latex_config += r' \\'
         metric = '\\text{' + re.split(r'[UD]', symbol)[0] + '}'
@@ -1673,7 +1697,7 @@ class Parser:
                 RHS += '^{%s}_{%s %s} (%s)' % (index, bound_index, covdrv_index, latex)
             else:
                 RHS += '^{%s}_{%s %s} (%s)' % (bound_index, index, covdrv_index, latex)
-        config = (' % assign -diff_type=dD \'' + symbol + '\'') if symbol else ''
+        config = (' % assign -diff_type=dD ' + symbol + '\n') if symbol else ''
         return LHS + ' = ' + RHS + config
 
     @staticmethod
